/*****************************************************************************
Programmation de la PMI - BTS 2000


Fichier .h general
Regroupe tous les definitions necessaires a la compilation du codage
complet du robot PMI.


Programme par OLLIVIER & CHARNIAUX
*****************************************************************************/



#include "c:\temp\yxia\pit.h"

typedef unsigned char boolean;
typedef unsigned char bit;

/*---------------------------------------------------------------------------*\
|	DEFINES
\*---------------------------------------------------------------------------*/

#define MAX	100

/* Temps d'immobilisation en cas de choc (en secondes). */
#define TPSCASCHOC 5

/* Codes de retour */
#define OK 0
#define CHOC -1
#define OVERFLOW -1
#define VIRDROITE 1
#define VIRGAUCHE 2
#define CARREFOUR 3
#define ALIGNEMENT 4
#define PERTELIGNE 5

#define LIGNEDROITE 1
#define ATTENTEVIR	2
#define ALIGN		3

#define ETAGMODPRECIS 0

#define tailleTableau 10 
#define nbPasPalier 6
#define valeurInfinie 9999
#define uneSeconde 250000 /* Valeur de precha PI/T pour 1 sec */

#define VRAI 1
#define FAUX 0
/* Commande de moteurs */
#define AVANT 1
#define ARRIERE 0
#define HALF 1
#define FULL 0
#define GAUCHE 0
#define DROITE 1

/* Possibilit de direction */
#ifndef GAUCHE
#define GAUCHE 0
#endif
#ifndef DROITE
#define DROITE 1
#endif
#define DEVANT 2
#define DEMIT 3
#define NBPOSSIBL 9
#define NBRESULT 2

/* N bit de port de controle des drivers */
#define	sensDriverD		0
#define sensDriverG		1
#define hfDriverD		2
#define hfDriverG		3
#define driverD  		4
#define driverG  		5
#define chocAv			6
#define chocAr			7

/* Indicateurs de limites */
#define INFINI 1500
#define LIGNES 10
#define COLONNES 10
#define SPEMAT 10
#define NBCHARS 20

#define NBPOINTMAX 20
#define INDICFINTRAME -1
#define ENDTAB -1
#define NULL 0

/* N vecteur de l' it */
#define vecteur	0x50



/*---------------------------------------------------------------------------*\
|	MACRO
\*---------------------------------------------------------------------------*/
#define CHAR_TO_NOMBRE(x) (x = x & 0x0F)


/*---------------------------------------------------------------------------*\
|	DECLARATION DES STRUCTURES   
\*---------------------------------------------------------------------------*/

typedef struct {

boolean sens;
/* boolean modePrecis;    /* VRAI active le mode precis DIRECTEMENT ! */
/* boolean suiviLigne;    /* VRAI active le suivi de ligne */
boolean deplacmtChoc;  /* VRAI active le gestion du deplacement par choc */

boolean choc;          /* Vrai aprs un choc */
boolean chocReperage;  /* Vrai quand un choc est apparut et que
						  deplacmtChoc etait VRAI */

boolean stateVirage;   /* Vrai si en etat virage */
boolean stateParcours; /* Vrai si en etat parcours */

boolean perteLigne;

boolean direction;     /* Quand VRAI une re-evaluation de la direction
						  est ncessaire.
                          (le pointeur de trame doit etre mis a jour) */


boolean assistance;    /* VRAI si on veux que les virages soit effectues
						par suiviLigne. */
boolean L_overflow;    /* Vrai si la liste des points a rejoindre a debordee */

}statePMI;


typedef struct {

	boolean modePrecis;	
	boolean sens;
	boolean suiviLigne;
	boolean statique;
	short tpsStat;		/* temps en secondes */
}mode_s;


typedef struct {

	unsigned long *etagValeurTimer;	/* Pointeur sur tableau de valeur d' acceleration */
	char nbEtages;		/* Nb etages d' accel, decel a faire */
	char noEtage;		/* Num d' etage de depart */
	unsigned long pasVitessConst;
}pas_s;


typedef struct {

	char virage;
	short angle;
}virage_s;

typedef struct {
	
	 boolean moteurD;
	 boolean moteurG;
	 boolean sensMoteurD;
	 boolean sensMoteurG;
}etatMoteurs_s;

/*---------------------------------------------------------------------------*\
|	Les structures parcours comprennent les informations necessaire a la
|	realisation d' une ligne droite ainsi que le type de virage a effectuer. 
\*---------------------------------------------------------------------------*/
typedef struct 
{
	short distance;	
	mode_s mode;
	pas_s  pas;
/*	virage_s *virage */
}parcours;

/*------------------------------------------------------------------------------*\
|	Les structures actions comprennent les informations necessaire a la
|	realisation de : virage droite, gauche, avec un angle precis ...
|	Elles sont stockees dans un tableau dont l' indice servira a les differencier.   
\*------------------------------------------------------------------------------*/
typedef struct
{

	boolean assistance;   /* Si VRAI c'est suiviLigne qui assiste le virage */
	etatMoteurs_s mot;
	short pasMoteurs;
	short noEtage;
	unsigned long *etagValeurTimer;

}actions;



/*---------------------------------------------------------------------------*\
|	DECLARATION DE NOUVEAUX TYPES POUR DIRCTION.C
\*---------------------------------------------------------------------------*/
typedef struct {
char virage;
char parcours;
}trame;

typedef struct {

char pointPrecdt;
char pointPosble;
char virage;
char parcours;
}s_posblt;


typedef struct {
s_posblt pos[NBPOSSIBL];
}vertex;


/*---------------------------------------------------------------------------*\
|	DEFINITION DES FONCTIONS
\*---------------------------------------------------------------------------*/

extern it();
extern setvect(int, void );
extern print(char*, ...);

short maj_Liste (char tabPoints[], char *ajt, char *courant, char *findata);
short gererMoteurs (parcours *pt1, actions *pt2);

short avancer (	long precharg,
				unsigned long nbPasEffect,
				boolean suiviLigne,
				boolean moteurD,
				boolean moteurG,
				octet etat );

void timer (boolean fonction, long valeurPrecharg);
void portA(char nobit, boolean bit);
void initChemin( char pointR[] );
void initValeurTimer( unsigned long ValeurTimer[] );
short casChoc (statePMI *etatPMI);
void actionVirage (actions *ptr, boolean assistance);

void plusCourtChemin (char pointAct,
					  char pointRej,
					  short matriceW[COLONNES][LIGNES],
					  char  matriceT[COLONNES][LIGNES][NBCHARS],
					  char newPts[] );

void initMatrice(short or_matriceW[COLONNES][LIGNES],
				 char  or_matriceT[COLONNES][LIGNES][NBCHARS]);

void direction (char, char, char, vertex[], trame *);
void represTable (vertex v[]);

int Suivi(octet etatpmi);
void ligne(octet d_vald);
int virage(octet d_vald);
int alignement(octet d_vald);

