/****************************************************************************
Programmation de la PMI - BTS 2000

Code de commande des moteurs avec gestion du timer 68230.

Version fonctionnelle - Revision du 21/06/2000


Programme par OLLIVIER Vincent
***************************************************************************/


#include "c:\temp\yxia\general.h"


pit_s *pit1;

/*****************************************************************************
* Nom de la fonction: gererMoteurs                                           *
*                                                                            *
* Utilite: Fait avancer le robot soit en ligne droite soit en virage         *
*          avec acceleration, deceleration et vitesse constante.             *
*                                                                            *
* Valeur de renvoi: -1 choc, 0 ok, et >0 (en fonction de Suivi(). )          *
*                                                                            *
* Parametres:  Entree : pointeur sur struct parcours ou actions.             *
*              Sortie : Aucuns                                               *
*                                                                            *
* Globales  :  Entree : Aucunes                                              *
*              Sortie : Aucunes                                              *
*                                                                            *
* Locales principales : int i, int result.                                   *
******************************************************************************/
short gererMoteurs (parcours *pt1, actions *pt2)
{
 int i, result;
 long *storePas;


	storePas = (long*)0x1000; 
	pit1=(pit_s*)0x60000; 

	/*---------------------------------*\
	|	Fixe le Half/Full step   
	\*---------------------------------*/    
 	portA(hfDriverD,HALF);
	portA(hfDriverG,HALF);

	/*--------------------------------------------------------------*\
	|	si pt1 est different de NULL on prend les donnees de
	|	la structure parcours donc on realise une ligne droite.    
	\*--------------------------------------------------------------*/    
   if(pt1 != NULL)
   {
	/*--------------------------------------------------------------*\
	|	On reste stationnaire sur carrefour si demande.
	\*--------------------------------------------------------------*/ 	   
	if(pt1->mode.statique == VRAI)
	{

		  portA(driverD,FAUX);
		  portA(driverG,FAUX);

	      *storePas = pt1->mode.tpsStat;            
		  timer(VRAI,uneSeconde);

	      while(*storePas >= 0){}

	      timer(FAUX,0);
    }

	/*---------------------------------------------------------------*\
	|	On fixe le sens de la marche (avant-arriere).
	\*---------------------------------------------------------------*/    
	portA(sensDriverD,pt1->mode.sens);
	portA(sensDriverG,pt1->mode.sens);

	/*---------------------------------------------------------------*\
	|	 A c c e l e r a t i o n   
	\*---------------------------------------------------------------*/ 
   if( pt1->pas.nbEtages != 0)
   {
	for(i=(pt1->pas.noEtage); i<= (pt1->pas.nbEtages); i++)
	{	
	result = avancer(pt1->pas.etagValeurTimer[i], 
					 nbPasPalier, pt1->mode.suiviLigne, 
					 VRAI, 
					 VRAI, 
					 LIGNEDROITE);
	
	if(result != 0){ return result; }
	}
   }

   	/*---------------------------------------------------------------*\
	|	 V i t e s s e   c o n s t a n t e   
	\*---------------------------------------------------------------*/ 
   if(pt1->pas.pasVitessConst != 0)
   {
	result = avancer(pt1->pas.etagValeurTimer[pt1->pas.nbEtages], 
					 pt1->pas.pasVitessConst, 
					 pt1->mode.suiviLigne, 
					 VRAI, 
					 VRAI, 
					 LIGNEDROITE);
	
	if(result != 0){ return result; }
   }


	/*---------------------------------------------------------------*\
	|	 D e c e l e r a t i o n   
	\*---------------------------------------------------------------*/    
   if( pt1->pas.nbEtages != 0)
   {
	for(i=(pt1->pas.nbEtages);  i>=(pt1->pas.noEtage);  i--)
	{
	result = avancer(pt1->pas.etagValeurTimer[i], 
					 nbPasPalier, 
					 pt1->mode.suiviLigne, 
					 VRAI, 
					 VRAI, 
					 LIGNEDROITE);
	
	if(result != 0){ return result; }
	}
   }



     if( (pt1->mode.suiviLigne == VRAI) && (pt1->mode.modePrecis == VRAI) ) 
     {  

	/*---------------------------------------------------------------*\
	|	 M o d e   p r e c i s   
	\*---------------------------------------------------------------*/  
	/* On attend de trouver un carrefour ! */
	result = avancer(pt1->pas.etagValeurTimer[ETAGMODPRECIS],
					 valeurInfinie,
					 VRAI,
					 VRAI,
					 VRAI, 
					 ATTENTEVIR);
	return result; 
     }	


   } /* Fin du si pt1 != NULL  */


	/*---------------------------------------------------------------*\
	|	Si le pt2 est different de NULL alors on prend les
	|	donnees de la structure actions pour realiser une action.
	\*---------------------------------------------------------------*/ 	
   if(pt2 != NULL)
   {
      portA(sensDriverD, pt2->mot.sensMoteurD);
      portA(sensDriverG, pt2->mot.sensMoteurG);

       result = avancer(pt2->etagValeurTimer[pt2->noEtage],
			pt2->pasMoteurs, 
			pt2->assistance, 
			pt2->mot.moteurD, 
			pt2->mot.moteurG, 
			ALIGN);

      if(result != 0){ return result; }
      
   }
   

return 0;

}









/*******************************************************************
* Nom de la fonction: portA                                        *
*                                                                  *
* Utilite: Arme  0 ou 1 un bit precis du portA.                   *
*                                                                  *
* Valeur de renvoi: Aucune.                                        *
*                                                                  *
* Parametres:  Entree : Numero du bit a armer, valeur de ce bit.   *
*              Sortie : Aucuns                                     *
*                                                                  *
* Globales  :  Entree : la structure du PI/T.                      *
*              Sortie : le registre PADR du PI/T.                  *
*                                                                  *
* Locales principales : Aucunes.                                   *
********************************************************************/

void portA(char nobit, boolean valeur)
{

if(valeur == 1)	pit1->PADR = pit1->PADR | (0x1<<nobit);
else	pit1->PADR = pit1->PADR & (unsigned char)~(0x1<<nobit);

}






/******************************************************************************
* Nom de la fonction: avancer                                                 *
*                                                                             *
* Utilite: autorise it, lance le timer, gere le deplacement                   *
*       (pas effect, choc...) arrete le timer, desactive it.                  *
*                                                                             *
* Valeur de renvoi: 0 ok, -1 choc, positif -> provient de Suivi().            *
*                                                                             *
* Parametres:  Entree : valeur de precharg pour timer, nombre pas a effectuer,*
*                       suiviLigne (VRAI/FAUX), moteur D et G (VRAI/FAUX),    *
*						etat de fonctionnement.                               *
*              Sortie : Aucuns                                                *
*                                                                             *
* Globales  :  Entree : Aucunes                                               *
*              Sortie : Aucunes                                               *
*                                                                             *
* Locales principales : int retour,result. char choc.                         *
*******************************************************************************/
short avancer (long precharg, 
			   unsigned long nbPasEffect, 
			   boolean suiviLigne, 
			   boolean moteurD, 
			   boolean moteurG, 
			   octet etat)
{

short retour=0, result=0;
long *storePas;

	storePas = (long*)0x1000; 

	/*------------------------------*\
	|	MAJ nombre pas effectues.
	\*------------------------------*/    
	*storePas = nbPasEffect;

	/*------------------------------*\
	|	E/D drivers moteurs
	\*------------------------------*/    
	portA(driverD,moteurD);
	portA(driverG,moteurG);

	/*------------------------------*\
	|	Demarrage timer
	\*------------------------------*/    
	timer(VRAI,precharg);
		

	/*--------------------------------------------------------------------------*\
	|	Boucle principale qui attend la fin du nombre de pas a effectuer.
	\*--------------------------------------------------------------------------*/ 
	/****************************************************************************
	 Avec ce code on peut savoir pourquoi on est sorti de la boucle
	 contrairement a : while( (choc != 0) && (result !=0) && ...    
	 Cette boucle est la section la plus critique du programme, on veux au moins
	 1 passage dans cette boucle entre 2 fronts de timer !
	 De plus une interruption apparait a chaque front ! 
	 Donc, le 68008 n' etant pas particulierement rapide, la solution des
	 breaks (jumps inconditionnels) semble la plus economique en terme
	 de temps processeur.
	*****************************************************************************/
      while(1)
      {
	    /* if( choc == VRAI)   { retour = -1; break; } */
	    if( *storePas <= 0) { retour =  0; break; }
            if( suiviLigne == VRAI)
	    {
	           result =  Suivi(etat) ; 
            }

            if((suiviLigne == VRAI) && (result != 0 )) {retour = result; break; } 
      }


	/*------------------------------*\
	|	Arret timer
	\*------------------------------*/    
	timer(FAUX,0); 	


return retour;
}






/******************************************************************************
* Nom de la fonction: timer                                                   *
*                                                                             *
* Utilite: Gestion du timer : demarrage, arret, chargement des registres.     *
*                                                                             *
* Valeur de renvoi: Aucune.                                                   *
*                                                                             *
* Parametres:  Entree : boolean fonction (VRAI, FAUX), valeurPrecharg         *
*						(valeur de prechargement).                            *
*              Sortie : Aucuns                                                *
*                                                                             *
* Globales  :  Entree : Aucunes                                               *
*              Sortie : Aucunes                                               *
*                                                                             *
* Locales principales : pit_s.                                                *
*******************************************************************************/
void timer (boolean fonction, long valeurPrecharg)
{
	
	if(fonction == VRAI)
	{
		pit1->CPRH = valeurPrecharg >> 16;
		pit1->CPRM = valeurPrecharg >> 8;
		pit1->CPRL = valeurPrecharg;
		pit1->TCR = (pit1->TCR) | 0x01;    /* Active le timer. */
	}

	if(fonction == FAUX)
	{
		pit1->TCR = (pit1->TCR) & 0xFE;	/* Desactive le timer. */
	}
}







